function [d, p_value] =  dcorr(X, Y, num_runs)
% [dcorr, pvalue] =  dcorr(X, Y)
%
% Computes the distance correlation between two datasets, where X
% and Y can be different dimensionalities.
%
% INPUT:
%   X: p x N, where p is the number of variables and N is the number of
%       samples
%   Y: q x N, where q is the number of variables and N is the number of
%       samples
%   num_runs (optional): integer, number of runs to compute the sample null
%       distribution; default = 1,000 runs
%
% OUTPUT:
%   dcorr: scalar between 0 and 1, the distance correlation
%   pvalue: scalar between 0 and 1, probability to accept the null 
%       hypothesis that X and Y are independent.  Computed by random
%       permutation test (i.e., randomly permuting Y to break any
%       dependence).
%
% Author:
%   bcowley, 10/16/2014

    if (nargin == 2)
        num_runs = 1000;
    end


    % compute the dcorr of the original data
    d = dcorr_plain(X,Y);
    
    % compute distribution of shuffled data (random permutation test)
    dcorr_shuffled = [];
    for irun = 1:num_runs
        r = randperm(size(Y,2));
        Yshuff = Y(:,r);
        
        dcorr_shuffled = [dcorr_shuffled dcorr_plain(X, Yshuff)];
    end
    
    % compute p_value
    p_value = sum(dcorr_shuffled > d) / num_runs;
    if (p_value == 0)
        p_value = 1/num_runs;
    end


end

function d = dcorr_plain(X,Y)
% helper function

    N = size(X,2);
    if (N ~= size(Y,2)) % Y has different number of trials
        error('Error: X and Y have different number of samples');
    end
    p = size(X,1);
    q = size(Y,1);
    

    % using the formula to compute the sample distance covariance
    
    % compute the euclidean distance matrices
    a = pdist2(X', X');  % a is N x N
    b = pdist2(Y', Y');

    % compute the row and column means
    a_kbar = mean(a,2);
    a_lbar = mean(a,1);
    a_bar = mean(a_lbar);
    b_kbar = mean(b,2);
    b_lbar = mean(b,1);
    b_bar = mean(b_lbar);
    
    % compute the adjusted matrices, A and B
    A = a - repmat(a_kbar, 1, N) - repmat(a_lbar, N, 1) + a_bar*ones(size(a));
    B = b - repmat(b_kbar, 1, N) - repmat(b_lbar, N, 1) + b_bar*ones(size(b));
    

    % compute the sample distance covariance
    dCov = sqrt(1/N^2 * sum(sum(A .* B)));
    
    % compute the sample distance variances
    dvarX = sqrt(1/N^2 * sum(sum(A .* A)));
    dvarY = sqrt(1/N^2 * sum(sum(B .* B)));
    
    % compute the sample distance correlation
    d = dCov / sqrt(dvarX * dvarY);


end