%% Examples of distance covariance
%
% Examples correspond to Simulations section in Scribed Notes
%
% We walk through the procedure to compute the sample distance covariance
% for three examples: a bi-variate non-linear dependency, independent
% Gaussian random vectors, and dependent random vectors with clusters.
%
% bcowley
% Oct. 27, 2014



%% Example 1

% idea:
% Let X = -10:0.1:10
% Then Y = X^2 + \epsilon, where \epsilon is small additive Gaussian noise
% generate data for X
%
% Can we understand what the matrices of computing the distance covariance
% mean?

% generate data for X
X = -10:0.1:10;
N = size(X,2);

% generate data for Y
Y = X.^2 + 5*randn(size(X));


% plot X and Y
f = figure;

plot(X,Y, 'b.');
hold on;
plot(X,X.^2, 'r-');
xlabel('X');
ylabel('Y');
saveas(f, 'figs/parabola_XY.eps', 'epsc');




% compute a_kl and b_kl, the distance matrix
a_kl = dist(X);
b_kl = dist(Y);

f = figure;
imagesc(a_kl, [0 20]);
colorbar;
xlabel('a_{kl}');
saveas(f, 'figs/parabola_a_kl.eps', 'epsc');
f = figure;
imagesc(b_kl, [0 115]);
colorbar;
xlabel('b_{kl}');
saveas(f, 'figs/parabola_b_kl.eps', 'epsc');


% compute a_kbar and b_kbar
a_kbar = mean(a_kl, 2);
b_kbar = mean(b_kl, 2);

f = figure;
imagesc(a_kbar, [0 20]);
xlabel('a_k');
saveas(f, 'figs/parabola_a_k.eps', 'epsc');

f = figure;
imagesc(b_kbar, [0 115]);
xlabel('b_k');
saveas(f, 'figs/parabola_b_k.eps', 'epsc');

% compute a_lbar and b_lbar
a_lbar = mean(a_kl, 1);
b_lbar = mean(b_kl, 1);

f = figure;
imagesc(a_lbar, [0 20]);
xlabel('a_l');
saveas(f, 'figs/parabola_a_l.eps', 'epsc');

f = figure;
imagesc(b_lbar, [0 115]);
xlabel('b_l');
saveas(f, 'figs/parabola_b_l.eps', 'epsc');



% compute a_bar and b_bar
a_bar = mean(a_lbar);
b_bar = mean(b_lbar);

f = figure;
imagesc(a_bar, [0 20]);
xlabel('a_{bar}');
saveas(f, 'figs/parabola_a_bar.eps', 'epsc');

f = figure;
imagesc(b_bar, [0 115]);
xlabel('b_{bar}');
saveas(f, 'figs/parabola_b_bar.eps', 'epsc');



% compute A_kl and B_kl
A_kl = a_kl - repmat(a_kbar,1,N) - repmat(a_lbar, N, 1) + a_bar * ones(N);
B_kl = b_kl - repmat(b_kbar,1,N) - repmat(b_lbar, N, 1) + b_bar * ones(N);
f = figure;
imagesc(A_kl);
colorbar;
xlabel('A_{kl}');
saveas(f, 'figs/parabola_Abig_kl.eps', 'epsc');
f = figure;
imagesc(B_kl);
colorbar;
xlabel('B_{kl}');
saveas(f, 'figs/parabola_Bbig_kl.eps', 'epsc');



% compute A_kl .* B_kl and V_n^2
V = 1/N^2 * A_kl .* B_kl;
V_n2 = sum(sum(V));
f = figure;
imagesc(V);
colorbar;
xlabel(sprintf('A_{kl} .* B_{kl}, V_n^2 = %0.3f', V_n2));
saveas(f, 'figs/parabola_A_kl_B_kl.eps', 'epsc');




%% Example 2

% idea:
% See how the different matrices look for computing the distance covariance
% with three clusters
%
% data:
% X: three standard normal figs/clusters at [0,5], [5,5], [10, 5]
% Y: also three figs/clusters of standard normals correlated with X
%       at [5,0], [5,5], [5,10]
% Thus, X and Y are correlated (just permuted the figs/clusters)
%
% Compute:
% a_kl, the distance matrix
% a_kbar, the mean of the rows (the center of all figs/clusters)
% a_lbar, the mean of the columns (population mean)
% a_bar, the mean of the all elements in the matrix
%
% then,
% A_kl represents the residuals to calculate the variance of the distances
%
% in a similar manner, B_kl is computed
%
% Then,
% V_n^2(X,Y) = 1/n^2 \sum_kl=1^n Akl Bkl


% generate data for X
x1 = bsxfun(@plus, randn(2,10), [0; 5]);
x2 = bsxfun(@plus, randn(2,10), [5; 5]);
x3 = bsxfun(@plus, randn(2,10), [10; 5]);
X = [x1 x2 x3];

% generate data for Y
y1 = bsxfun(@plus, randn(2,10), [5; 0]);
y2 = bsxfun(@plus, randn(2,10), [5; 5]);
y3 = bsxfun(@plus, randn(2,10), [5; 10]);
Y = [y1 y2 y3];


% plot X and Y
f = figure;

plot(x1(1,:), x1(2,:), '.r');
hold on;
plot(x2(1,:), x2(2,:), '.g');
plot(x3(1,:), x3(2,:), '.b');
title('X');
xlim([-5 15]);
ylim([-5 15]);
saveas(f, 'figs/clusters_X.eps', 'epsc');


f = figure;
plot(y1(1,:), y1(2,:), '.r');
hold on;
plot(y2(1,:), y2(2,:), '.g');
plot(y3(1,:), y3(2,:), '.b');
title('Y');
xlim([-5 15]);
ylim([-5 15]);
saveas(f, 'figs/clusters_Y.eps', 'epsc');



% compute a_kl and b_kl, the distance matrix
a_kl = dist(X);
b_kl = dist(Y);

f = figure;
imagesc(a_kl, [0 15]);
colorbar;
xlabel('a_{kl}');
saveas(f, 'figs/clusters_a_kl.eps', 'epsc');
f = figure;
imagesc(b_kl, [0 15]);
colorbar;
xlabel('b_{kl}');
saveas(f, 'figs/clusters_b_kl.eps', 'epsc');




% compute a_kbar and b_kbar
a_kbar = mean(a_kl, 2);
b_kbar = mean(b_kl, 2);

f = figure;
imagesc(a_kbar, [0 15]);
xlabel('a_k');
saveas(f, 'figs/clusters_a_k.eps', 'epsc');

f = figure;
imagesc(b_kbar, [0 15]);
xlabel('b_k');
saveas(f, 'figs/clusters_b_k.eps', 'epsc');

% compute a_lbar and b_lbar
a_lbar = mean(a_kl, 1);
b_lbar = mean(b_kl, 1);

f = figure;
imagesc(a_lbar, [0 15]);
xlabel('a_l');
saveas(f, 'figs/clusters_a_l.eps', 'epsc');

f = figure;
imagesc(b_lbar, [0 15]);
xlabel('b_l');
saveas(f, 'figs/clusters_b_l.eps', 'epsc');



% compute a_bar and b_bar
a_bar = mean(a_lbar);
b_bar = mean(b_lbar);

f = figure;
imagesc(a_bar, [0 15]);
xlabel('a_{bar}');
saveas(f, 'figs/clusters_a_bar.eps', 'epsc');

f = figure;
imagesc(b_bar, [0 15]);
xlabel('b_{bar}');
saveas(f, 'figs/clusters_b_bar.eps', 'epsc');



% compute A_kl and B_kl
A_kl = a_kl - repmat(a_kbar,1,30) - repmat(a_lbar, 30, 1) + a_bar * ones(30);
B_kl = b_kl - repmat(b_kbar,1,30) - repmat(b_lbar, 30, 1) + b_bar * ones(30);
f = figure;
imagesc(A_kl);
colorbar;
xlabel('A_{kl}');
saveas(f, 'figs/clusters_Abig_kl.eps', 'epsc');
f = figure;
imagesc(B_kl);
colorbar;
xlabel('B_{kl}');
saveas(f, 'figs/clusters_Bbig_kl.eps', 'epsc');



% compute A_kl .* B_kl and V_n^2
V = 1/30^2 * A_kl .* B_kl;
V_n2 = sum(sum(V));
f = figure;
imagesc(V);
colorbar;
xlabel(sprintf('A_{kl} .* B_{kl}, V_n^2 = %0.3f', V_n2));
saveas(f, 'figs/clusters_A_kl_B_kl.eps', 'epsc');



%% Example 3
% now repeat for two standard normals

% generate data for X
X = randn(2,30);

% generate data for Y
Y = randn(2,30);


% plot X and Y
f = figure;
plot(X(1,:), X(2,:), '.r');
title('X');
xlim([-5 15]);
ylim([-5 15]);
saveas(f, 'figs/gaussian_X.eps', 'epsc');

f = figure;
plot(Y(1,:), Y(2,:), '.r');
title('Y');
xlim([-5 15]);
ylim([-5 15]);
saveas(f, 'figs/gaussian_Y.eps', 'epsc');



% compute a_kl and b_kl, the distance matrix
a_kl = dist(X);
b_kl = dist(Y);

f = figure;
imagesc(a_kl, [0 25]);
colorbar;
xlabel('a_{kl}');
saveas(f, 'figs/gaussian_a_kl.eps', 'epsc');
f = figure;
imagesc(b_kl, [0 25]);
colorbar;
xlabel('b_{kl}');
saveas(f, 'figs/gaussian_b_kl.eps', 'epsc');



% compute a_kbar and b_kbar
a_kbar = mean(a_kl, 2);
b_kbar = mean(b_kl, 2);

f = figure;
imagesc(a_kbar, [0 25]);
xlabel('a_k');
saveas(f, 'figs/gaussian_a_k.eps', 'epsc');
f = figure;
imagesc(b_kbar, [0 25]);
xlabel('b_k');
saveas(f, 'figs/gaussian_b_k.eps', 'epsc');

% compute a_lbar and b_lbar
a_lbar = mean(a_kl, 1);
b_lbar = mean(b_kl, 1);

f = figure;
imagesc(a_lbar, [0 25]);
xlabel('a_l');
saveas(f, 'figs/gaussian_a_l.eps', 'epsc');
f = figure;
imagesc(b_lbar, [0 25]);
xlabel('b_l');
saveas(f, 'figs/gaussian_b_l.eps', 'epsc');




% compute a_bar and b_bar
a_bar = mean(a_lbar);
b_bar = mean(b_lbar);

f = figure;
imagesc(a_bar, [0 25]);
xlabel('a_{bar}');
saveas(f, 'figs/gaussian_a_bar.eps', 'epsc');
f = figure;
imagesc(b_bar, [0 25]);
xlabel('b_{bar}');
saveas(f, 'figs/gaussian_b_bar.eps', 'epsc');


% compute A_kl and B_kl
A_kl = a_kl - repmat(a_kbar,1,30) - repmat(a_lbar, 30, 1) + a_bar * ones(30);
B_kl = b_kl - repmat(b_kbar,1,30) - repmat(b_lbar, 30, 1) + b_bar * ones(30);
f = figure;
imagesc(A_kl);
colorbar;
xlabel('A_{kl}');
saveas(f, 'figs/gaussian_Abig_kl.eps', 'epsc');
f = figure;
imagesc(B_kl);
colorbar;
xlabel('B_{kl}');
saveas(f, 'figs/gaussian_Bbig_kl.eps', 'epsc');




% compute A_kl .* B_kl and V_n^2
V = 1/30^2 * A_kl .* B_kl;
V_n2 = sum(sum(V));
f = figure;
imagesc(V);
colorbar;
xlabel(sprintf('A_{kl} .* B_{kl}, V_n^2 = %0.3f', V_n2));
saveas(f, 'figs/gaussian_A_kl_B_kl.eps', 'epsc');












